@@ -10,6 +10,7 @@ import datetime
 import logging
 import boto3
 import argparse
+import re
 
 from models import NetworkInterface
 from models import bcolors
@@ -25,8 +26,12 @@ def parse_arguments() -> dict:
              description="Find public endpoinds in AWS")
     parser.add_argument("--loglevel",
                         help="Set logging level")
-    parser.add_argument("--console", action="store_const", const=True)
-    parser.add_argument("--csv", action="store_const", const=True)
+    parser.add_argument("--console", action="store_const", const=True,
+                        help="Write output to console.")
+    parser.add_argument("--csv", action="store_const", const=True,
+                        help="Write output to file.")
+    parser.add_argument("--filename",
+                        help="Output filename. Default is FindMyEndpoints-output-[date].csv")  # noqa: E501
     parser.add_argument("--organization", action="store_const", const=True,
                         help="Check all accounts in an Organization")
     parser.add_argument("--controltower", action="store_const", const=True,
@@ -128,7 +133,7 @@ def get_regions() -> list:
     return regions
 
 
-def process_accounts(regions: str,
+def process_accounts(regions: list,
                      account_id: str,
                      credentials=None,
                      default_region=None) -> dict:
@@ -217,12 +222,21 @@ def process_network_interfaces(boto3_session: boto3.Session,
                 instance_info = get_network_interface_attachment(
                                     ec2_resource,
                                     nic.id)
-                nic.instance_id = instance_info.get("InstanceId")
-                instance_tag_info = get_instance_name(
-                                        ec2_client,
-                                        nic.instance_id)
-                instance_tags = instance_tag_info.get("Tags")
-                nic.instance_name = instance_tags[0].get("Value")
+                if not re.match("amazon-aws|amazon-rds|amazon-redshift|amazon-elb",
+                                instance_info.get("InstanceOwnerId")):
+                    nic.instance_id = instance_info.get("InstanceId")
+                    instance_tag_info = get_instance_name(
+                                            ec2_client,
+                                            nic.instance_id)
+                    instance_tags = instance_tag_info.get("Tags")
+                    nic.instance_name = f"(EC2) {instance_tags[0].get('Value')}"
+                elif re.match("amazon-elb", instance_info.get("InstanceOwnerId")):
+                    network_interface_desc = network_interface.get("Description").split('/')
+                    nic.instance_name = f"(ELB) {network_interface_desc[1]}"
+                    nic.instance_id = f"(ELB) {network_interface_desc[1]}"
+                else:
+                    nic.instance_name = instance_info.get("InstanceOwnerId")
+                    nic.instance_id = instance_info.get("InstanceOwnerId")
                 nic.account_id = account_id
                 yield nic
 
@@ -369,21 +383,25 @@ def assume_role_in_org_account(session: boto3.Session,
         boto3 session object
     """
     sts_client = get_boto3_client(session, "sts", region)
-    response = sts_client.assume_role(
-        RoleArn="arn:aws:iam::{}:role/{}"
-        .format(account_id, role_name),
-        RoleSessionName="FindMyEndpoints"
-    )
     credentials = {}
-    credentials["AccessKeyId"] = response \
-                        .get("Credentials") \
-                        .get("AccessKeyId")
-    credentials["SecretAccessKey"] = response \
-                        .get("Credentials") \
-                        .get("SecretAccessKey")
-    credentials["SessionToken"] = response \
-                        .get("Credentials") \
-                        .get("SessionToken")
+    try:
+        response = sts_client.assume_role(
+            RoleArn="arn:aws:iam::{}:role/{}"
+            .format(account_id, role_name),
+            RoleSessionName="FindMyEndpoints"
+        )
+        credentials["AccessKeyId"] = response \
+                            .get("Credentials") \
+                            .get("AccessKeyId")
+        credentials["SecretAccessKey"] = response \
+                            .get("Credentials") \
+                            .get("SecretAccessKey")
+        credentials["SessionToken"] = response \
+                            .get("Credentials") \
+                            .get("SessionToken")
+    except Exception as e:
+        credentials = None
+        logging.error(e)
     return credentials
 
 
@@ -404,23 +422,31 @@ def output_console(network_interfaces):
                                   bcolors.bcolors.DEFAULT))
         for region in network_interfaces.get(account_id).keys():
             if len(network_interfaces.get(account_id).get(region)) > 0:
+                print("{}\n{}{}".format(bcolors.bcolors.YELLOW,
+                                        region,
+                                        bcolors.bcolors.DEFAULT))                
                 for i in network_interfaces.get(account_id).get(region):
-                    print("{}\n{}{}".format(bcolors.bcolors.YELLOW,
-                                            region,
-                                            bcolors.bcolors.DEFAULT))
                     print("{}Network Interface Id\tInstance Id\t\tInstance Name\tPublic Ip\tPublic DNS{}"  # noqa: E501
                           .format(bcolors.bcolors.LIGHTGRAY,
                           bcolors.bcolors.DEFAULT))
                     if i.region == region:
-                        print("{}\t{}\t{}\t{}\t{}".format(
-                            i.id,
-                            i.instance_id,
-                            i.instance_name,
-                            i.public_ip,
-                            i.public_dns))
+                        if len(i.instance_id) >= 16:
+                            print("{}\t{}\t{}\t{}\t{}".format(
+                                i.id,
+                                i.instance_id,
+                                i.instance_name,
+                                i.public_ip,
+                                i.public_dns))
+                        else:
+                            print("{}\t{}\t\t{}\t{}\t{}".format(
+                                i.id,
+                                i.instance_id,
+                                i.instance_name,
+                                i.public_ip,
+                                i.public_dns))                            
 
 
-def output_csv(network_interfaces):
+def output_csv(network_interfaces, filename=None):
     """
     Writes output to a csv
 
@@ -431,7 +457,10 @@ def output_csv(network_interfaces):
         None
     """
     time_stamp = datetime.datetime.now().strftime("%Y%m%d-%H%M%S")
-    output_filename = "FindMyEndpoints_output-{}.csv".format(time_stamp)
+    if filename is not None:
+        output_filename = filename
+    else:
+        output_filename = "FindMyEndpoints-output-{}.csv".format(time_stamp)
     try:
         f = open(output_filename, "w")
         f.write("Account Id,Network Interface Id,Instance Id,Instance Name,Public Ip,Public DNS,Region\n")  # noqa: E501
@@ -464,6 +493,10 @@ def main():
         if not isinstance(numeric_level, int):
             raise ValueError('Invalid log level: %s' % loglevel)
         logging.basicConfig(level=numeric_level)
+    if vars(args).get("filename"):
+        filename = vars(args).get("filename")
+    else:
+        filename = None
     regions = []
     regions = get_regions()
     if vars(args).get("default-region"):
@@ -487,6 +520,9 @@ def main():
                     exit(1)
                 credentials = assume_role_in_org_account(mgmt_account_session,
                                                          "us-east-1", account_id, role_name)  # noqa: E501
+                if credentials is None:
+                    logging.error("Unable to assume role in account")
+                    exit(1)
                 network_interfaces[account_id] = process_accounts(regions,
                                                                   account_id,
                                                                   credentials=credentials,  # noqa: E501
@@ -496,11 +532,11 @@ def main():
     else:
         sts_client = boto3.client("sts")
         account_id = sts_client.get_caller_identity().get("Account")
-        network_interfaces[account_id] = process_accounts(regions)
+        network_interfaces[account_id] = process_accounts(regions, account_id)
     if vars(args).get("console"):
         output_console(network_interfaces)
     if vars(args).get("csv"):
-        output_csv(network_interfaces)
+        output_csv(network_interfaces, filename=filename)
     return
 
 
